'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' A demo using libcurl.dll functions to download
' several URLs to local files using threads.
' ---
'	Libcurl is a free and easy-to-use client-side
' URL transfer library, supporting
' FTP, FTPS, HTTP, HTTPS, GOPHER, TELNET,
' DICT, FILE and LDAP.
' ---
' See CURL:
' http://curl.haxx.se/libcurl/
'
PROGRAM	"multithread"
VERSION	"0.0001"
CONSOLE
'
	IMPORT	"xst"   		' Standard library : required by most programs
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT 	"libcurl"		' CURL library

TYPE URL_DATA
	STRING * 256 .url
	STRING * 256 .file
END TYPE
'
DECLARE FUNCTION  Entry ()
DECLARE CFUNCTION  WriteData (bufAddr, elemSize, numElem, fileNumber)
DECLARE FUNCTION  PullOneURLThread (dataAddr)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
'
FUNCTION  Entry ()

	URL_DATA data[]

	curl_global_init ($$CURL_GLOBAL_WIN32)	' initialize CURL library

	DIM data[3]
	DIM hThread[3]
	DIM threadID[3]

	data[0].url = "https://curl.haxx.se/libcurl/"
	data[1].url = "https://curl.haxx.se/libcurl/c/libcurl-tutorial.html"
	data[2].url = "https://curl.haxx.se/libcurl/c/10-at-a-time.html"
	data[3].url = "https://raw.githubusercontent.com/curl/curl/master/docs/examples/10-at-a-time.c"

	data[0].file = "libcurl.htm"
	data[1].file = "libcurl-tutorial.html"
	data[2].file = "10-at-a-time.html"
	data[3].file = "10-at-a-time.c"

	start = GetTickCount ()

	FOR i = 0 TO 3
		hThread[i] = CreateThread (NULL, 0, &PullOneURLThread(), &data[i], 0, &threadID[i])
		IFZ hThread[i] THEN
			PRINT "Couldn't run thread number"; i; " : error number :"; GetLastError()
		ELSE
			PRINT "Thread "; i; " gets "; data[i].url
		END IF
	NEXT i

' wait for all thread to finish
	ret = WaitForMultipleObjects (4, &hThread[], $$TRUE, $$INFINITE)

	finish = GetTickCount () - start
	PRINT
	PRINT "Elapsed time ="; finish; " msec"
	PRINT

	FOR i = 0 TO 3
		GetExitCodeThread (hThread[i], &return)
		PRINT "Thread"; i; " : GetExitCodeThread : return ="; return
		CloseHandle (hThread[i])
	NEXT i

	curl_global_cleanup ()								' clean up CURL library

	a$ = INLINE$ ("Press ENTER to exit >")

END FUNCTION
'
'
' ##########################
' #####  WriteData ()  #####
' ##########################
'
CFUNCTION  WriteData (bufAddr, elemSize, numElem, fileNumber)

'	PRINT bufAddr, elemSize, numElem, fileNumber

	size = elemSize * numElem
	data$ = NULL$ (size)
	RtlMoveMemory (&data$, bufAddr, size)

	' append file
	SEEK (fileNumber, LOF (fileNumber))
	WRITE [fileNumber], data$

	RETURN size

END FUNCTION
'
'
' #################################
' #####  PullOneURLThread ()  #####
' #################################
'
FUNCTION  PullOneURLThread (dataAddr)

	URL_DATA data

 	curl_handle = curl_easy_init ()														'  init the curl session

	IF curl_handle THEN

		RtlMoveMemory (&data, dataAddr, SIZE (data))						' copy data into struct

		url$ = data.url																					' download an html url
		file$ = data.file

		curl_easy_setopt (curl_handle, $$CURLOPT_URL, &url$)		' set URL to get
		curl_easy_setopt (curl_handle, $$CURLOPT_NOPROGRESS, 1)	' no progress meter please
		curl_easy_setopt (curl_handle, $$CURLOPT_MUTE, 1)				' shut up completely
		curl_easy_setopt (curl_handle, $$CURLOPT_WRITEFUNCTION, &WriteData()) ' send all data to this FUNCTION
		curl_easy_setopt (curl_handle, $$CURLOPT_SSL_VERIFYPEER, 0) ' do not verify ssl certificate

 		ofile = OPEN (file$, $$RWNEW)							' open the file to write data
 		IF (ofile < 3) THEN
			PRINT "File OPEN error"
			curl_easy_cleanup (curl_handle)
			RETURN ($$TRUE)
		END IF

 		curl_easy_setopt (curl_handle, $$CURLOPT_WRITEDATA, ofile)	' we want the data to this file handle
		curl_easy_perform (curl_handle) 				' get the file
		PRINT "Download done on URL "; url$
		PRINT "File size :"; LOF (ofile); " bytes."
		CLOSE (ofile)														' close the data file
		curl_easy_cleanup (curl_handle)					'	cleanup curl stuff
	END IF


END FUNCTION
END PROGRAM
